@@ -4,7 +4,7 @@ use utf8;
 use strict;
 use warnings FATAL => 'all';
 
-our $VERSION = "0.03";
+our $VERSION = "0.04";
 
 require Exporter;
 our @ISA = qw(Exporter);
@@ -13,17 +13,19 @@ our @EXPORT =
 
 use Encode qw//;
 
+#=========================================
 #@category Кодировка
 
-# Определяет кодировку. 
-# В koi8-r и в cp1251 большие и малые буквы как бы поменялись местами, поэтому у правильной кодировки вес будет больше
+# Определяет кодировку.
 sub bohemy($) {
     my ($s) = @_;
     my $c = 0;
-    while ( $s =~ /[а-яё]+/gi ) {
-        my $x = $&;
-        if   ( $x =~ /^[А-ЯЁа-яё][а-яё]*$/ ) { $c += length $x }
-        else                                 { $c -= length $x }
+    while ( $s =~ m![а-яё]+!gi ) {
+        my $len = length $&;
+        if    ( $& =~ /^[А-ЯЁ][а-яё]+$/ ) { $c += $len }
+        elsif ( $& =~ /^[А-ЯЁ]+$/ )       { $c += $len / 3 }
+        elsif ( $& =~ /^[а-яё]+$/ )       { $c += $len / 3 }
+        else                              { $c -= $len }
     }
     $c;
 }
@@ -61,6 +63,7 @@ sub detect(@) {
     ( decode @_ )[1];
 }
 
+#=========================================
 #@category Вспомогательные функции
 
 # Ищет файлы в директориях рекурсивно
@@ -97,7 +100,7 @@ sub file_write(@) {
       or die "При открытии для записи $file произошла ошибка: $!.";
     print $f $unicode;
     close $f;
-	return;
+    return;
 }
 
 # Определить кодировку и декодировать файл
@@ -109,14 +112,68 @@ sub file_decode(@) {
 # Кодировать в указанной кодировке и записать в файл
 sub file_encode(@) {
     my ( $file, $encoding, $unicode ) = @_;
-	
-	utf8::decode($unicode) if !utf8::is_utf8($unicode);
+
+    utf8::decode($unicode) if !utf8::is_utf8($unicode);
 
     $unicode = Encode::encode( $encoding, $unicode ) if defined $encoding;
 
     file_write $file, $unicode;
 }
 
+#=========================================
+#@category Тестирование файлов
+
+# Если расширение отсутствует, то 1-я строка должна содержать #!(.*)$interpreter
+# * $exts — расширения: [qw/pl pm py/]
+# * $interpreters — интерпретаторы: [qw/perl python/]
+# Примечание: $interpreters проверяются только для файлов без расширений и только если пустое расширение было в $exts
+sub test_file(@) {
+    my ( $file, $exts, $interpreters ) = @_;
+
+    $exts         //= [];
+    $interpreters //= [];
+
+    my ($ext) = $file =~ /\.([^.\/]*)$/;
+
+    return @$exts ? ( 1 == grep { $ext eq $_ } @$exts ) : 1 if defined $ext;
+
+    open my $f, "<", $file or die "$file: $!";
+    read $f, my $buf, 2;
+    close($f), return 0 if $buf ne "#!";
+
+    return 1 if !@$interpreters;
+
+    my $first_line = <$f>;
+    close $f;
+
+    $interpreters = join "|", @$interpreters;
+
+    $first_line =~ /.*\b($interpreters)\b/ ? 1 : 0;
+}
+
+# Тестирует файлы на соответствия расширениям
+sub test_files(@) {
+    my ( $files, $exts, $interpreters ) = @_;
+
+    $exts         = [ split /,/, $exts ]         if !ref $exts;
+    $interpreters = [ split /,/, $interpreters ] if !ref $interpreters;
+
+    grep test_file( $_, $exts, $interpreters ), @$files;
+}
+
+# Возвращает изменённые файлы в репозитории git
+sub change_files() {
+    map { file_find $_ }
+      map { s/^\s*[\w\?]+\s+//; $_ } grep { !/^\s*D / } split /\n/,
+      `git status -s`;
+}
+
+# Возвращает изменённые файлы в ветке
+sub change_files_in_branch() {
+    grep length, split "\n",
+      `git diff --name-only --diff-filter=AM origin/master...`;
+}
+
 1;
 __END__
 
@@ -124,7 +181,7 @@ __END__
 
 =head1 NAME
 
-Octets::To::Unicode - модуль утилит для распознавания кодировки текста (в том числе в файлах) и его декодирования
+Octets::To::Unicode - модуль и утилиты ru-perltidy и ru-utf8 для распознавания кодировки текста (в том числе в файлах) и его декодирования.
 
 =head1 VERSION
 
@@ -145,7 +202,7 @@ Octets::To::Unicode - модуль утилит для распознавани
 	file_encode "path/to/file2", "koi8-r", $file_text_in_unicode;
 
 Использование утилит:
-	
+
 	# Отформатировать указанные файлы perltidy:
 	$ ru-perltidy file1 file2
 
@@ -160,11 +217,23 @@ Octets::To::Unicode - модуль утилит для распознавани
 
 	# Указать расширения файлов:
 	$ ru-perltidy --ext 'pl,pm,'
+	
+	# Обработать файлы в директориях:
+	$ ru-perltidy --in-dir .,/tmp/mydir
 
-	# Перевести файлы во временные в кодировке utf-8 (в /tmp) и после выполнения команды и их изменения переписать обратно в определён
-	ной кодировке:
-	# (тут $1 - первый файл, $2 - второй и т.д., $* - все файлы через пробел. Так же работают подстановки ${1} и т.д.)
-	$ ru-utf8 file1 file2 -c 'perltidy $1 -st > $2'
+	# Выполнить операцию с файлами:
+	$ ru-utf8 file1 file2 -c 'perltidy $f -st > $o'
+	
+	# Переменные, которые можно использовать:
+	$ ru-utf8 file1 file2 -c 'echo $f $o $e $x'
+	$ ru-utf8 file1 file2 -o -c 'echo $f1 $o1 $e1 $x1 - $f2 $o2 $e2 $x2'
+	
+	# Кроме команды шелла можно использовать ещё код perl:
+	$ ru-utf8 file1 file2 -s 'print "$f $o $e $x -- $unicode\n"'
+	$ ru-utf8 file1 file2 -o -s 'print "@f @o @e @x"'
+	
+	# Определить кодировку файлов и перекодировать их в koi8-r:
+	$ ru-encoding -t koi8-r
 
 =head1 DESCRIPTION
 
@@ -176,6 +245,8 @@ Octets::To::Unicode - модуль утилит для распознавани
 
 =item B<ru-utf8> — переводит файлы во временные (в кодировке utf-8), выполняет указанную команду и переписывает обратно в определённой кодировке;
 
+=item B<ru-encoding> — перекодирует файлы в указанную кодировку.
+
 =back
 
 и модуль perl:
@@ -362,11 +433,79 @@ Cписок кодировок, которыми предлагается поп
 
 =back
 
+=head2 test_file
+
+    $is_file = test_file $file, $exts, $interpreters;
+
+Тестирует файл на соответствие указанным расширениям, а если расширения нет, то на соответсвие интерпретаторов к указанному в первой строке файла начинающейся на C<#!>.
+
+Принимает параметры:
+
+=over 4
+
+=item B<$file>
+
+Путь к файлу.
+
+Обязательный.
+
+=item B<$exts>
+
+Список расширений для сопоставления, если он пуст, то подходит любое.
+
+Необязательный. Значение по умолчанию: C<[]>.
+
+=item B<$interpreters>
+
+Список интерпретаторов для сопоставления, если он пуст, то подходит любой, главное, чтобы строка начиналась на C<#!>.
+
+Необязательный. Значение по умолчанию: C<[]>.
+
+=back
+
+=head2 test_files
+
+    @files = test_files $files, $exts, $interpreters;
+
+Тестирует файлы на соответствие указанным расширениям или интерпретаторам.
+
+Принимает параметры:
+
+=over 4
+
+=item B<$files>
+
+Список файлов. 
+
+Обязательный.
+
+=item B<$exts>
+
+Такой же как в B<test_file>.
+
+=item B<$interpreters>
+
+Такой же как в B<test_file>.
+
+=back
+
+=head2 change_files
+
+    @files = change_files();
+
+Возвращает изменённые файлы в репозитории git.
+
+=head2 change_files
+
+    @files = change_files_in_branch();
+
+Возвращает изменённые файлы в ветке.
+
 =head1 INSTALL
 
 Установить можно любым менеджером C<perl> со B<CPAN>, например:
 
-	$ cpm install -g Octets::To::Unicode
+	$ sudo cpm install -g Octets::To::Unicode
 
 =head1 DEPENDENCIES
 
@@ -404,9 +543,9 @@ Cписок кодировок, которыми предлагается поп
 
 	$ git pull
 	
-=item Отредактировать файл __Changes__.
+=item Отредактировать файл I<Changes>.
 
-В файле __Changes__ нужно написать список изменений, которые вошли в этот релиз.
+В файле I<Changes> нужно написать список изменений, которые вошли в этот релиз.
 
 Изменения записываются в виде списка, одно изменение — один элемент списка. Элементы списка обозначаются символами тире `-`.
 
@@ -423,13 +562,13 @@ Cписок кодировок, которыми предлагается поп
 	 
 		- RU-4 Какой-то предыдущий тикет.
 
-	Обратите внимание — у нового релиза пока нет версии. Версия будет вычислена Миниллой при выполнении релиза и автоматически вписана в файл CHANGES вместо метки C<{{$NEXT}}>.
+Обратите внимание — у нового релиза пока нет версии. Версия будет вычислена Миниллой при выполнении релиза и автоматически вписана в файл I<Changes> вместо метки C<{{$NEXT}}>.
 
 =item Активировать локальную библиотеку:
 
 	$ cpanm --local-lib=~/perl5 local::lib && eval $(perl -I ~/perl5/lib/perl5/ -Mlocal::lib)
 
-	Это нужно, чтобы не выполнять релиз под рутом.
+Это нужно, чтобы не выполнять релиз под рутом.
 
 =item Выполнить релиз:
 
@@ -441,6 +580,14 @@ Cписок кодировок, которыми предлагается поп
 
 =back
 
+=head1 LINKS
+
+=over 4
+
+=item * perltidy и cp1251 / L<https://habr.com/ru/post/664308/>.
+
+=back
+
 =head1 AUTHOR
 
 Yaroslav O. Kosmina E<lt>darviarush@mail.ruE<gt>
